/*
 * Functions for combining density matrices and for tracing out sub-systems from
 * full-system density matrices.
 * 
 * 
 * Robert Johansson <robert@riken.jp>
 *
 * $Id: density_matrix.h,v 1.2 2008-09-24 10:14:52 rob Exp $
 */

#ifndef DENSITY_MATRIX_H
#define DENSITY_MATRIX_H

#include <gsl/gsl_matrix.h>

#include "quantum_system.h"

typedef struct {
	gsl_matrix_complex *dm[NQS_MAX];
	int len;
} density_matrix_list_t;

int density_matrix_combine(gsl_matrix_complex *dm, quantum_system *qs, density_matrix_list_t *dm_list);
gsl_matrix_complex *density_matrix_traceout(quantum_system *qs, gsl_matrix_complex *dm, int n);
gsl_matrix_complex *density_matrix_traceout_multi(quantum_system *qs, gsl_matrix_complex *dm, quantum_system *mask);

void density_matrix_list_init(density_matrix_list_t *dm_list);
void density_matrix_list_append(density_matrix_list_t *dm_list, gsl_matrix_complex *dm);
void density_matrix_list_free_entries(density_matrix_list_t *dm_list);

gsl_complex density_matrix_expectation_value(gsl_matrix_complex *rho, gsl_matrix_complex *op);

gsl_matrix_complex *density_matrix_pure_TLS(double p_ex);
gsl_matrix_complex *density_matrix_fock_state(int n, int N);
gsl_matrix_complex *density_matrix_coherent_state(double r, double theta, int N, int offset);
gsl_matrix_complex *density_matrix_boson_thermal(double w, double w_th, int N);

gsl_vector_complex *wave_function_fock_state(int n, int N);
int wave_function_coherent_state(gsl_vector_complex *wf, double r, double theta, int N, int offset);
	
double density_matrix_concurrence(quantum_system *qs, gsl_matrix_complex *rho);
	

int density_matrix_mix(gsl_matrix_complex *rho_mix, gsl_matrix_complex *rho1, gsl_matrix_complex *rho2);
int wave_function_superposition(gsl_vector_complex *wf_sp, gsl_vector_complex *wf1, gsl_vector_complex *wf2);
int wave_function_to_density_matrix(gsl_matrix_complex *rho, gsl_vector_complex *wf);

gsl_complex density_matrix_trace(gsl_matrix_complex *rho);


gsl_complex wave_function_expectation_value(gsl_vector_complex *wf, gsl_matrix_complex *op);


#endif

