/*
 * 
 * Robert Johansson <robert@riken.jp>
 *
 * 
 */

#ifndef DIST_FUNCS_H
#define DIST_FUNCS_H

#include <gsl/gsl_integration.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>

#include "quantum_system.h"
#include "density_matrix.h"

int distribution_function_Q(gsl_matrix_complex *Q, gsl_matrix_complex *rho, double alpha_max);


int distribution_function_characteristic_w(
            gsl_matrix_complex *phase_space, 
            gsl_matrix_complex *a,
            gsl_matrix_complex *ad,
            gsl_matrix_complex *rho,
            gsl_complex lambda_max);

double cb_integrand_wigner_cf(gsl_complex lambda, gsl_matrix_complex *phase_space_cf, gsl_complex alpha); 

int distribution_function_wigner_cf_quad(
            gsl_matrix_complex *phase_space_wigner, 
            gsl_matrix_complex *a,
            gsl_matrix_complex *ad,
            gsl_matrix_complex *rho,
            gsl_complex alpha_max);

int distribution_function_wigner_cf_sum(
            gsl_matrix_complex *phase_space_wigner, 
            gsl_matrix_complex *a,
            gsl_matrix_complex *ad,
            gsl_matrix_complex *rho,
            gsl_complex alpha_max);

int distribution_function_wigner_cf_fft(
            gsl_matrix_complex *phase_space_wigner, 
            gsl_matrix_complex *a,
            gsl_matrix_complex *ad,
            gsl_matrix_complex *rho,
            gsl_complex alpha_max);

typedef struct _wigner_hermite_data {
    double p;
    double q;
    gsl_matrix_complex *rho;
    int N;
} wigner_hermite_data_t;

typedef struct _wigner_data {
    double p;
    double q;
 
    gsl_matrix_complex *Cw;

    gsl_complex lambda, alpha;
    gsl_complex lambda_max, alpha_max;

    gsl_matrix_complex *rho;

    gsl_function f_re, f_im;

    gsl_integration_workspace  *w_re, *w_im;
    gsl_integration_qawo_table *wf_re, *wf_im;

    int N;


} wigner_data_t;

double cb_integrand_wigner_hermite(double x, void *data);
int distribution_function_wigner_hermite_quad(
            gsl_matrix_complex *phase_space_wigner, 
            gsl_matrix_complex *rho,
            double x_limit);

#endif

