/**
 * @file hamiltonian.h
 *
 * @brief Functions for generating hamiltonians in matrix form. 
 * 
 * @author Robert Johansson <robert@riken.jp>
 *
 */

#ifndef HAMILTONIAN_H
#define HAMILTONIAN_H

#include <gsl/gsl_matrix.h>
#include "param.h"

typedef  int(*hamiltonian_func_real_t)(quantum_system *qs, solver_param *param, gsl_matrix_complex *H);

int hamiltonian_t(gsl_matrix_complex *H_t, gsl_matrix_complex *H0, gsl_matrix_complex *H1, double t, solver_param *p);

int hamiltonian_nTLS(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_z_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_x_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_2ho(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_2ho_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_q2ho(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_q2ho_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_2ho_optmech(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_2ho_cooling(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_qho(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_qho_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_3ls_ho(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_3ls_ho_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_ho(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);
int hamiltonian_ho_drive(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_dicke(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

int hamiltonian_squeezing(gsl_matrix_complex *H, quantum_system *qs, solver_param *param);

#endif

