/*
 * Functions for integration of systems of differential equations. 
 * 
 * 
 * Robert Johansson <robert@riken.jp>
 *
 * $Id: integrate.h,v 1.1.1.1 2008-09-03 09:36:41 rob Exp $
 */

#ifndef INTEGRATE_H
#define INTEGRATE_H

#include "param.h"
#include "master_equation.h"

#include <gsl/gsl_matrix.h>

#define RK_REC_MAX_LEVEL 15

typedef int (*ode_func_drho_dt)(gsl_matrix_complex *rho_t, gsl_matrix_complex *drho_dt, double t, 
			  quantum_system *qs, solver_param *p);

int odeint_complex_rk(quantum_system *qs, solver_param *p, gsl_matrix_complex *rho0, 
		      ode_func_drho_dt f_drho_dt, rho_cb_func_t rho_cb_func);

int odeint_complex_rk_adaptive(quantum_system *qs, solver_param *p, gsl_matrix_complex *rho0, 
		               ode_func_drho_dt f_drho_dt, rho_cb_func_t rho_cb_func);

int odeint_rk_adaptive_rec(quantum_system *qs, solver_param *p, ode_func_drho_dt f_drho_dt,
		           gsl_matrix_complex *rho0, gsl_matrix_complex *rho_final,
		           int level, double ti, double tf, double etol, double *e);

double max_deviation(gsl_matrix_complex *m1, gsl_matrix_complex *m2, double etol);
	

#endif

