/*
 * Functions for generating matrix representations of quantum operators.
 * 
 * 
 * Robert Johansson <robert@riken.jp>
 *
 * $Id: operators.h,v 1.3 2008-09-11 10:00:57 rob Exp $
 */

#ifndef OPERATORS_H
#define OPERATORS_H

#include <gsl/gsl_matrix.h>

int operator_sigma_z(gsl_matrix_complex *op, quantum_system *qs, int n);
int operator_sigma_x(gsl_matrix_complex *op, quantum_system *qs, int n);
int operator_sigma_y(gsl_matrix_complex *op, quantum_system *qs, int n);

int operator_3ls_sigma_z(gsl_matrix_complex *op, quantum_system *qs, int n);
int operator_3ls_sigma_x(gsl_matrix_complex *op, quantum_system *qs, int n);

int operator_J_minus(gsl_matrix_complex *op, quantum_system *qs, int n, double jj);
int operator_J_plus (gsl_matrix_complex *op, quantum_system *qs, int n, double jj);
int operator_Jz     (gsl_matrix_complex *op, quantum_system *qs, int n, double jj);

int operator_unit(gsl_matrix_complex *op, quantum_system *qs, int n);

int operator_ho_lowering(gsl_matrix_complex *op, quantum_system *qs, int n, int offset);
int operator_ho_raising(gsl_matrix_complex *op, quantum_system *qs, int n, int offset);
int operator_ho_N(gsl_matrix_complex *op, quantum_system *qs, int n, int offset);

int operator_project(gsl_matrix_complex *op, quantum_system *qs, int n, int si, int sj);
	
gsl_complex operator_trace(gsl_matrix_complex *op);


#endif

