/*
 * Test matrix read and write to file operations.
 *
 *
 * JRJ, <robert@riken.jp>
 */
#include <stdio.h> 
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_cblas.h>
#include <gsl/gsl_blas.h>

#include "misc.h"

int
main(int argc, char **argv)
{
	char *filename = "matrix.dat";
	int i, j, n = 10;
	gsl_matrix_complex *orig, *copy;

	srand(time(NULL));

	orig = gsl_matrix_complex_alloc(n,n);
	copy = gsl_matrix_complex_alloc(n,n);

	/*
	 * Create random matrix:
	 *
	 */
	for (i = 0; i < n; i++)
	{
		for (j = 0; j < n; j++)
		{
			gsl_complex z;
			GSL_SET_COMPLEX(&z, rand() / 128.0, rand() / 128.0);
			gsl_matrix_complex_set(orig, i, j, z);
		}
	}
	
	printf("orig = \n");
	misc_matrix_complex_print_real(orig, n);
		

	/*
	 * Write matrix to file:
	 */ 
	misc_matrix_write(filename, orig, n);

	misc_matrix_read(filename, copy, n);

	printf("copy = \n");
	misc_matrix_complex_print_real(copy, n);

	/*
	 * Compare matrices:
	 *
	 */
	for (i = 0; i < n; i++)
	{
		for (j = 0; j < n; j++)
		{
			gsl_complex zo, zc;

			zo = gsl_matrix_complex_get(orig, i, j);
			zc = gsl_matrix_complex_get(copy, i, j);

			if ( fabs(GSL_REAL(zo) - GSL_REAL(zc)) > 0.0001 ||
			     fabs(GSL_IMAG(zo) - GSL_IMAG(zc)) > 0.0001)
			{
				printf("Error: readback missmatch at location [%d, %d]\n", i, j);
			}
		}
	}

	/*
	 * Write matrix to file:
	 */ 
	misc_matrix_diag_write(filename, orig, n, "");

	misc_matrix_diag_read(filename, copy, n);

	printf("copy = \n");
	misc_matrix_complex_print_real(copy, n);

	/*
	 * Write matrix to file:
	 */ 
	misc_matrix_diag_real_write(filename, orig, n, "");

	misc_matrix_diag_real_read(filename, copy, n);

	printf("copy = \n");
	misc_matrix_complex_print_real(copy, n);


	return 0;
	
	
	/*
	 * Compare matrices:
	 *
	 */
	for (i = 0; i < n; i++)
	{
		for (j = 0; j < n; j++)
		{
			gsl_complex zo, zc;

			zo = gsl_matrix_complex_get(orig, i, j);
			zc = gsl_matrix_complex_get(copy, i, j);

			if ( fabs(GSL_REAL(zo) - GSL_REAL(zc)) > 0.0001 ||
			     fabs(GSL_IMAG(zo) - GSL_IMAG(zc)) > 0.0001)
			{
				printf("Error: readback missmatch at location [%d, %d]\n", i, j);
			}
		}
	}

	printf("Test finished.\n");
	
	return 0;
}
